import com.threed.jpct.*;
import java.util.*;

public class CameraManager{
	
	private final static float CAMERA_ZOOM_MIN = 10;
	private final static float CAMERA_ZOOM_MAX = 2000;
	
	private final static float CAMERA_ANGLE_MIN = .001f;
	private final static float CAMERA_ANGLE_MAX = 3f;


	public final static float ADJUST_SPEED = 2;
	public final static float ADJUST_SPEED_PI = .025f;
	public final static float ADJUST_SPEED_ANGLE = .005f;

	
	public boolean zoom_in = false;
	public boolean zoom_out = false;
	public boolean angle_up = false;
	public boolean angle_down = false;

	private float _z_factor = 0.73f;
	private float _zoom = 900f;

	private World _world = null;
	private Vector _vsO = null;
		   

   	public CameraManager(World w){
		_world = w;
		_vsO = new Vector();
	}

	public void addObject3D(Object3D o){
		try{
			if(_vsO == null)
				_vsO = new Vector();
			
			_vsO.addElement(o);

		}catch(Exception e){
			System.err.println("CameraManager::addObject3D() " + e.getMessage());
			e.printStackTrace();
		}
	}
   
	private Object3D getTarget(){
		Object3D o = null;
		try{
			
			if(_vsO != null){
				Object3D ofirst = null;
				for(int i=0; i < _vsO.size(); i++){
					o = (Object3D)_vsO.elementAt(i);
					if(i==0)
						break;
				}
			}

		}catch(Exception e){
			System.err.println("CameraManager::getTarget() " + e.getMessage());
			e.printStackTrace();
		}
		return o;
	}

   public void checkModifySettings(){
		try{
			if(zoom_out){
				if(_zoom + ADJUST_SPEED < CAMERA_ZOOM_MAX)
					_zoom += ADJUST_SPEED;
			}
			if(zoom_in){
				if(_zoom - ADJUST_SPEED > CAMERA_ZOOM_MIN)
					_zoom-=ADJUST_SPEED;
			}
			if(angle_up){
				if(_z_factor < CAMERA_ANGLE_MAX)
					_z_factor+=ADJUST_SPEED_ANGLE;
			}

			if(angle_down){
				if(_z_factor > CAMERA_ANGLE_MIN)
					_z_factor-=ADJUST_SPEED_ANGLE;
			}

			//if(angle_up || angle_down || zoom_in || zoom_out)
			//	System.out.println("CameraManager::checkModifySettings() zoom:z_factor " + _zoom + ":" + _z_factor);

		}catch(Exception e){
			System.err.println("CameraManager::checkModifySettings() " + e.getMessage());
			e.printStackTrace();
		}
	}


	public void update(FocalPoint fp){
		moveCamera();
	}

	private void moveCamera() {
		try{
			Object3D target = getTarget();

			SimpleVector oldCamPos=_world.getCamera().getPosition();
			SimpleVector oldestCamPos=new SimpleVector(oldCamPos);

			oldCamPos.scalarMul(9f);
			
			SimpleVector targetCenter=target.getTransformedCenter();
			SimpleVector camPos=new SimpleVector(targetCenter);
			SimpleVector zOffset=target.getZAxis();
			
			SimpleVector yOffset=new SimpleVector(0, _zoom * -1, 0);
			zOffset.scalarMul(_zoom * -1 * _z_factor);

			camPos.add(zOffset);

			
			camPos.add(yOffset);

			camPos.add(oldCamPos);
			
			camPos.scalarMul(0.1f);

			SimpleVector delta=camPos.calcSub(oldestCamPos);
			float len=delta.length();

			if (len!=0) {
				_world.checkCameraCollisionEllipsoid(delta.normalize(), new SimpleVector(20, 20, 20), len, 3);
			}

			_world.getCamera().lookAt(target.getTransformedCenter());

		}catch(Exception e){
			System.err.println("CameraManager::moveCamera() " + e.getMessage());
			e.printStackTrace();
		}
	}
}