import java.awt.*;
import org.lwjgl.input.*;
import java.nio.*;
import java.awt.image.*;
import java.io.*;
import javax.imageio.*;

public class CursorManager{

	public static final int MODE_NORMAL = 0;
	public static final int MODE_PLACE = 1;
	public static final int MODE_SELECT = 2;
	public static final int MODE_ROTATE = 3;
	public static final int MODE_MOVE = 4;
	public static final int MODE_NO = 5;
	public static final int MODE_CAMERA = 6;

	public static final String CUR_NORMAL_GL = "arrowgl.gif";
	public static final String CUR_NORMAL = "arrow.gif";

	public static final String CUR_PLACE_GL = "placegl.gif";
	public static final String CUR_PLACE = "place.gif";

	public static final String CUR_SELECT_GL = "selectgl.gif";
	public static final String CUR_SELECT = "select.gif";

	public static final String CUR_ROTATE_GL = "rotategl.gif";
	public static final String CUR_ROTATE = "rotate.gif";

	public static final String CUR_MOVE_GL = "movegl.gif";
	public static final String CUR_MOVE = "move.gif";

	public static final String CUR_NO_GL = "cantgl.gif";
	public static final String CUR_NO = "cants.gif";

	public static final String CUR_CAMERA_GL = "cameragl.gif";
	public static final String CUR_CAMERA = "camera.gif";

	public int lastmode = MODE_NORMAL;
	public int mode = MODE_NORMAL;

	org.lwjgl.input.Cursor _nativeCursor = null;

	public void updateNativeCursor(){
		try{
			if(_nativeCursor != null)
				Mouse.setNativeCursor(_nativeCursor);
		}catch(Exception e){
			System.err.println("CursorManager.updateNativeCursor() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public CursorManager(boolean bogl, Frame f){
		setMode(bogl, f, MODE_NORMAL);
	}

	private String getCursorMode(Frame f, boolean bogl){
		String s = CUR_NORMAL;
		try{
			switch(mode){
				case(MODE_NORMAL):{	
					if(f != null && bogl == false)
						s = CUR_NORMAL;
					else
						s = CUR_NORMAL_GL;
					break;
				}
				case(MODE_PLACE):{	
					if(f != null && bogl == false)
						s = CUR_PLACE;
					else
						s = CUR_PLACE_GL;
					break;
				}
				case(MODE_SELECT):{	
					if(f != null && bogl == false)
						s = CUR_SELECT;
					else
						s = CUR_SELECT_GL;
					break;
				}
				case(MODE_ROTATE):{
					if(f != null && bogl == false)
						s = CUR_ROTATE;
					else
						s = CUR_ROTATE_GL;
					break;
				}
				case(MODE_MOVE):{
					if(f != null && bogl == false)
						s = CUR_MOVE;
					else
						s = CUR_MOVE_GL;
					break;
				}
				case(MODE_NO):{
					if(f != null && bogl == false)
						s = CUR_NO;
					else
						s = CUR_NO_GL;
					break;
				}
				case(MODE_CAMERA):{
					if(f != null && bogl == false)
						s = CUR_CAMERA;
					else
						s = CUR_CAMERA_GL;
					break;
				}

			}
			s = DevkitConfig.getInstance().getValue(DevkitConfig.DEVKIT_TEXTURES_PATH) + "/" + s;

		}catch(Exception e){
			System.err.println("CursorManager::getCursorMode() " + e.getMessage());
			e.printStackTrace();
		}
		return s;
	} 

	public void setMode(boolean bogl, Frame f, int iMode){
		try{

			lastmode = mode;
			mode = iMode;

			String scur = getCursorMode(f, bogl);


			if(f != null && bogl == false){
		        Toolkit toolkit = Toolkit.getDefaultToolkit();
		        BufferedImage cursorImage = ImageIO.read(getClass().getClassLoader().getResourceAsStream(scur));
				Point cursorHotSpot = new Point(0,0);
		        java.awt.Cursor customCursor = toolkit.createCustomCursor(cursorImage, cursorHotSpot, "x");
				f.setCursor(customCursor);
			}else{
				loadNativeCursor(scur);
			}
		}catch(Exception e){
			System.err.println("CursorManager::setCursor() " + e.getMessage());
			e.printStackTrace();
		}

	}

	private void loadNativeCursor(String sFile) throws Exception{
		try{

			//System.out.println("CursorManager::loadNativeCursor() loading: " + sFile);
        
			Image image = loadCursor(sFile);

	        _nativeCursor = new org.lwjgl.input.Cursor(32, 32, 0, 30, 1, createBuffer(image), null);

	        updateNativeCursor();

		}catch(Exception e){
			System.err.println("CursorManager::loadNativeCursor() " + e.getMessage());
			e.printStackTrace();
		}
	}


    private Image loadCursor(String sFile)
    {
        int i = 0x186a0;
        int j = 0;
        int k = 0;
        int l = i;
        byte abyte0[] = new byte[i];
        
		InputStream is = null;
		try{
			
			//System.out.println("CursorManager::loadCursor() loading file: " + sFile);
			is = getClass().getClassLoader().getResourceAsStream(sFile);
			        
		}catch(Exception fe){
			System.err.println("CursorManager::loadCursor() failed to load file! " + fe.getMessage());
			fe.printStackTrace();
		}
        
		Image image = null;
        try{
            do{
                if(j == -1){
                    break;
                }
                j = is.read(abyte0, k, l - k);
                k += j;
                if(j != -1 && k >= l){
                    l += 0x186a0;
                    byte abyte1[] = new byte[l];
                    System.arraycopy(abyte0, 0, abyte1, 0, l - 0x186a0);
                    abyte0 = abyte1;
                    System.out.println("Expanding buffers..." + l + " bytes");
                }
            } while(true);
            k++;
            is.close();
            image = Toolkit.getDefaultToolkit().createImage(abyte0, 0, k);
        }
        catch(Exception e){
            System.err.println("CusorManager::loadCursor() Couldn't get data from InputStream: " + e.toString());
			e.printStackTrace();
        }

        MediaTracker mediatracker = new MediaTracker(new Canvas());
        mediatracker.addImage(image, 0);
        try{
            mediatracker.waitForAll();
        }catch(Exception e1){
            System.err.println("CusorManager::loadCursor() Couldn't get data from InputStream: " + e1.toString());
			e1.printStackTrace();
        }

        return image;
    }

    private IntBuffer createBuffer(Image image)
    {
        int i = image.getHeight(null) * image.getWidth(null);
        ByteBuffer bytebuffer = ByteBuffer.allocateDirect(i << 2);
        bytebuffer.order(ByteOrder.LITTLE_ENDIAN);
        int ai[] = new int[i];
        PixelGrabber pixelgrabber = new PixelGrabber(
				image, 0, 0, image.getWidth(null), image.getHeight(null), ai, 0, image.getWidth(null));
        try{
            pixelgrabber.grabPixels();
        }
        catch(InterruptedException interruptedexception){
            System.err.println("CursorManager::createBuffer() Could not grab pixels from image!");
        }
        for(int j = 0; j < i; j++){
            int k = j << 2;
            int l = ai[j];
            if(l != 0){
                l |= 0xff000000;
            }
            bytebuffer.putInt(k, l);
        }

        return bytebuffer.asIntBuffer();
    }



}