import com.threed.jpct.*;
import com.threed.jpct.util.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;


public class InputManager {

	public boolean exit = false;
	public boolean fullscreen=false;
	public boolean openGL=false;
	public boolean bloom=false;
	public boolean wireframe=false;
    public int switchMode=0;

	public int height = 480;
	public int width = 640;	

    private boolean key_left=false;
    private boolean key_right=false;
    private boolean key_forward=false;
    private boolean key_jump=false;

    private float speed=0;

	public final static int SWITCH_RENDERER=35;
	
	private KeyMapper _keymap =null;
	private MouseMapper _mousemap = null;
	private CameraManager _cammgr = null;
	private GUIEventHandler _gui = null;
	private InputState _inputstate = null;
	private CursorManager _cursorMgr = null;	

	private Frame _frame = null;


	private boolean _initialized=false;

	public void updateGUI(){
		_gui.update();
	}

	public void initializeGUI(World w, FocalPoint fp){
		try{
			_gui = new GUIEventHandler();
			_gui.init(w, fp);			
			_gui.setCursorMgr(_cursorMgr);

		}catch(Exception e){
			System.err.println("InputManager::initializeGUI() " + e.getMessage());
			e.printStackTrace();
		}
	}




	public InputManager(){}

	public void checkMoveLight(){
		try{
			_gui.getObjectsLoader().checkMoveSelectedLight();

		}catch(Exception e){
			System.err.println("InputManager::checkMoveLight() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public boolean initialized(){return _initialized;}


	public void init(Frame f, CameraManager cm){
		try{
			
			_frame = f;
			_cammgr = cm;


	  		_keymap = new KeyMapper(_frame);
	  		_mousemap = new MouseMapper(_frame, height);

			_inputstate = new InputState();

			_cursorMgr = new CursorManager(false, f);

			_initialized = true;

		}catch(Exception e){
			System.err.println("InputManager::init() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public void destroy(){
		try{
			_frame = null;
			if(_keymap != null)
				_keymap.destroy();
			_keymap = null;
			
			if(_mousemap != null)
				_mousemap.destroy();
			_mousemap = null;
			
			_cursorMgr = null;
							

			_initialized = false;

		}catch(Exception e){
			System.err.println("InputManager::destroy() " + e.getMessage());
			e.printStackTrace();
		}
	}


	public void initOGL(){
		try{
			destroy();

			_frame = null;
			
            _keymap = new KeyMapper();

			_mousemap = new MouseMapper(height);
			_mousemap.show();
 
			_inputstate = new InputState();

			_cursorMgr = new CursorManager(true, null);

			System.out.println("InputManager::initOGL()...");

			_initialized = true;

		}catch(Exception e){
			System.err.println("InputManager::initOGL() " + e.getMessage());
			e.printStackTrace();
		}
	}



	public void processKeyboardMouse(FrameBuffer b, World w, FocalPoint fp){
		try{

			handleMouseAction(b, w);

			KeyState state=null;
			do{

				state = _keymap.poll();
				if(state != KeyState.NONE)
            		keyAffected(state, fp);

      		}while(state!=KeyState.NONE);

		}catch(Exception e){
			System.err.println("InputManager::processKeyboardMouse() " + e.getMessage());
			e.printStackTrace();
		}
	}

	private void keyAffected(KeyState state, FocalPoint fp) {
		try{
		
			int code=state.getKeyCode();
			boolean event=state.getState();

			switch (code) {
         		case (KeyEvent.VK_ESCAPE): {
            		exit=event;
            		break;
         		}

				case (KeyEvent.VK_HOME): {
					_cammgr.angle_down = event;
					break;
				}
				case (KeyEvent.VK_END): {
					_cammgr.angle_up = event;
					break;
				}

				case (KeyEvent.VK_PAGE_UP): {
					_cammgr.zoom_in = event;
					break;
				}
				case (KeyEvent.VK_PAGE_DOWN): {
					_cammgr.zoom_out = event;
					break;
				}
				case (KeyEvent.VK_UP): {
					if(!checkLightMove(KeyEvent.VK_UP, event))
						fp.forward = event;
					break;
				}
				case (KeyEvent.VK_DOWN): {
					if(!checkLightMove(KeyEvent.VK_DOWN, event))
						fp.backward = event;
					break;
				}
				case (KeyEvent.VK_F1): {
					if(!checkLightMove(KeyEvent.VK_F1, event))
						fp.up = event;
					break;
				}
				case (KeyEvent.VK_F2): {
					if(!checkLightMove(KeyEvent.VK_F2, event))
						fp.down = event;
					break;
				}
				case (KeyEvent.VK_LEFT): {
					if(!checkLightMove(KeyEvent.VK_LEFT, event))
						fp.turn_left = event;
					break;
				}
				case (KeyEvent.VK_RIGHT): {
					if(!checkLightMove(KeyEvent.VK_RIGHT, event))
						fp.turn_right = event;
					break;
				}
				default: {
					_gui.processKey();
				}
	  		}
		}catch(Exception e){
			System.err.println("InputManager::keyAffected() " + e.getMessage());
			e.printStackTrace();
		}
	}


	



   private void handleMouseAction(FrameBuffer buffer, World w){
		try{

			if(_mousemap != null){
				int x = _mousemap.getMouseX();
				int y = _mousemap.getMouseY();

				if(!_gui.mouseAction()){
					
			        int[] res=Interact2D.pickPolygon(
							w.getVisibilityList(), Interact2D.reproject2D3D(w.getCamera(), buffer, x, y));

			      	if (res!=null) {

			        	Object3D obj = w.getObject(Interact2D.getObjectID(res));	            		
						String sname = "<null>";
						if(obj != null)
							sname = obj.getName();

						SimpleVector vm = CalcHelper.getMouseVector(w, w.getCamera(), buffer, x, y);

						if(vm != null)
							_gui.updateMouseOverProperties(sname, vm.x, vm.y, vm.z);			

				   		if(_inputstate.stateChanged(_mousemap)){							
							_gui.checkToggleSelect(obj, _mousemap.buttonDown(MouseMapper.BUTTON1), _cammgr);
							_gui.checkPlaceMarker(vm, _mousemap.buttonDown(MouseMapper.BUTTON1), w);
							_gui.checkPlaceFocalPoint(vm, _mousemap.buttonDown(MouseMapper.BUTTON1), w);

						}

		      		}
					_cursorMgr.updateNativeCursor();

				}
			}else{
				System.err.println("InputManager::handleMouseAction() _mousemap is null!");
			}
		}catch(Exception e){
			System.err.println("InputManager::handleMouseAction() " + e.getMessage());
			e.printStackTrace();
		}
	}

	private boolean checkLightMove(int code, boolean event){
		boolean b = false;
		try{

			if(_gui.getObjectsLoader().isLightSelected()){	
				b = true;
				SimpleVector v = new SimpleVector(0f, 0f, 0f);
				if(code == KeyEvent.VK_UP) 
					_gui.getObjectsLoader().light_forward = event;
				if(code == KeyEvent.VK_DOWN)
					_gui.getObjectsLoader().light_backward = event;
				if(code == KeyEvent.VK_LEFT)
					_gui.getObjectsLoader().light_left = event;				
				if(code == KeyEvent.VK_RIGHT)
					_gui.getObjectsLoader().light_right = event;
				if(code == KeyEvent.VK_F1)
					_gui.getObjectsLoader().light_up = event;
				if(code == KeyEvent.VK_F2)
					_gui.getObjectsLoader().light_down = event;
			}

		}catch(Exception e){
			System.err.println("InputManager::checkFocalPointMove() " + e.getMessage());
			e.printStackTrace();
		}
		return b;
   }



}