import com.threed.jpct.*;
import com.threed.jpct.util.*;
import com.threed.jpct.procs.*;
import java.util.*;

public class PropertyChanger{
	
	private static final String VAL_TRUE = "true";
	private static final String VAL_FALSE = "false";


	public static void reconcileObjectProperties(World w, ObjectsLoader ol, HashMap hstmp, HashMap hs) throws Exception{
		try{
			
			Iterator ito = hstmp.values().iterator();
			while (ito.hasNext()) {
				ObjectAttributes oao = (ObjectAttributes)ito.next();
				
				Iterator iti = hs.values().iterator();
				String skey = oao.key;
				System.out.println("PropertyChanger::reconcileObjectProperties() looking to match [" + oao.key + "]");
				while(iti.hasNext()){
					ObjectAttributes oai = (ObjectAttributes)iti.next();
					if((oai.obj3D.getName().equals(skey) 
							|| oai.obj3D.getName().startsWith(skey))
								&& (oai.load_matched==false)){

						System.out.println("PropertyChanger::reconcileObjectProperties() matched [" + oao.key + "] to [" + oai.key + "]");
						
						oai.load_matched = true;
						oai.cloneValues(oao);
						
						applyObjectSettings(oai, w, ol);

						break;
					}
				}
			}

		}catch(Exception e){
			System.err.println("PropertyChanger::reconcileObjectProperties() " + e.getMessage());
			e.printStackTrace();
			throw e;
		}
	}
	
	private static void applyObjectSettings(ObjectAttributes oa, World w, ObjectsLoader ol) throws Exception{
		try{
			
			SimpleVector v = new SimpleVector(oa.x, oa.y, oa.z);
			SimpleVector vtc = oa.obj3D.getTransformedCenter();
			vtc.scalarMul(-1);
			oa.obj3D.translate(vtc);
			oa.obj3D.translate(v);
			
			oa.obj3D.rotateX(oa.x_rot);
			oa.obj3D.rotateY(oa.y_rot);
			oa.obj3D.rotateZ(oa.z_rot);
			
			oa.obj3D.setScale(oa.scale);
			
			oa.obj3D.setCulling(oa.culling);
			oa.obj3D.setTransparency(oa.transparency);

			System.out.println("PropertyChanger::applyObjectSettings() devkit_type is: " + oa.devkit_type);

			if(oa.devkit_type.equals(ObjectsLoader.DEVKIT_OBJECT_TYPE_LIGHT)){
				System.out.println("PropertyChanger::applyObjectSettings() altering light properties...");
				oa.light.setPosition(v);
				System.out.println("PropertyChanger::applyObjectSettings() attenuation " + oa.attenuation);
				oa.light.setAttenuation(oa.attenuation);
				oa.light.setIntensity(new SimpleVector(oa.intensity_r, oa.intensity_g, oa.intensity_b));
				oa.light.enable();
			}


			

		}catch(Exception e){
			System.err.println("PropertyChanger::applyObjectSettings() " + e.getMessage());
			e.printStackTrace();
			throw e;
		}
	}

	public static void updateScale(World w, ObjectsLoader ol, WidgetBuilder wb){
		try{
			String sname = wb.getSelectedList().getLabel().getText();
			if((!sname.equals(WidgetBuilder.NOTHING_SELECTED)) && (!sname.equals(""))){
				
				ObjectAttributes oa = ol.getObjectAttributes(w.getObjectByName(sname));
				float fval = Float.parseFloat(wb.getFieldText(WidgetBuilder.TE_OBJ_SCALE));
				oa.obj3D.setScale(fval);
				oa.scale = fval;
			}


		}catch(Exception e){
			System.err.println("PropertyChanger::updateTransparency() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public static void updateTransparency(World w, ObjectsLoader ol, WidgetBuilder wb){
		try{
			String sname = wb.getSelectedList().getLabel().getText();
			if((!sname.equals(WidgetBuilder.NOTHING_SELECTED)) && (!sname.equals(""))){
				
				ObjectAttributes oa = ol.getObjectAttributes(w.getObjectByName(sname));
				int ival = Integer.parseInt(wb.getFieldText(WidgetBuilder.TE_OBJ_TRANSPARENCY));
				oa.obj3D.setTransparency(ival);
				oa.transparency = ival;
			}


		}catch(Exception e){
			System.err.println("PropertyChanger::updateTransparency() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public static void updateCulling(World w, ObjectsLoader ol, WidgetBuilder wb){
		try{
			String sname = wb.getSelectedList().getLabel().getText();
			if((!sname.equals(WidgetBuilder.NOTHING_SELECTED)) && (!sname.equals(""))){
				
				ObjectAttributes oa = ol.getObjectAttributes(w.getObjectByName(sname));
				String sval = wb.getFieldText(WidgetBuilder.TE_OBJ_CULLING);

				boolean b = true;
				if(sval.compareToIgnoreCase(VAL_FALSE) == 0)
					b = false;

				oa.culling = b;
				oa.obj3D.setCulling(b);

			}


		}catch(Exception e){
			System.err.println("PropertyChanger::updateTransparency() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public static void updateLocation(World w, int imode, ObjectsLoader ol, WidgetBuilder wb){
		try{
			String sname = wb.getSelectedList().getLabel().getText();
			if((!sname.equals(WidgetBuilder.NOTHING_SELECTED)) && (!sname.equals(""))){
				
				ObjectAttributes oa = ol.getObjectAttributes(w.getObjectByName(sname));
				float fnew = 0f;	
							
				SimpleVector vtc = oa.obj3D.getTransformedCenter();
				SimpleVector v = new SimpleVector(vtc.x, vtc.y, vtc.z);

				if(wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_X){
					fnew = Float.parseFloat(wb.getFieldText(WidgetBuilder.TE_OBJ_X));
					v.x = fnew;
				}
				if(wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_Y){
					fnew = Float.parseFloat(wb.getFieldText(WidgetBuilder.TE_OBJ_Y));
					v.y = fnew;
				}
				if(wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_Z){
					fnew = Float.parseFloat(wb.getFieldText(WidgetBuilder.TE_OBJ_Z));
					v.z = fnew;
				}
				
				vtc.scalarMul(-1);
				oa.obj3D.translate(vtc);
				oa.obj3D.translate(v);

				oa.x = v.x;
				oa.y = v.y;
				oa.z = v.z;

			}
					
		}catch(Exception e){
			System.err.println("PropertyChanger::updateLocation() " + e.getMessage());
			e.printStackTrace();
		}
	}

	public static void updateRotation(World w, int imode, ObjectsLoader ol, WidgetBuilder wb){
		try{
			String sname = wb.getSelectedList().getLabel().getText();
			if((!sname.equals(WidgetBuilder.NOTHING_SELECTED)) && (!sname.equals(""))){
				
				ObjectAttributes oa = ol.getObjectAttributes(w.getObjectByName(sname));
				float fnew = 0f;	
							
				if(wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_X_ROT){
					fnew = Float.parseFloat(wb.getFieldText(WidgetBuilder.TE_OBJ_X_ROT));
					Float freverse = (oa.x_rot - oa.init_x_rot)*-1;
					oa.obj3D.rotateX(freverse);
					oa.obj3D.rotateX(fnew);
					oa.x_rot = fnew;
				}
				if(wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_Y_ROT){
					fnew = Float.parseFloat(wb.getFieldText(WidgetBuilder.TE_OBJ_Y_ROT));
					Float freverse = (oa.y_rot - oa.init_y_rot)*-1;
					oa.obj3D.rotateY(freverse);
					oa.obj3D.rotateY(fnew);
					oa.y_rot = fnew;
				}
				if(wb.getGeneralSetting() == WidgetBuilder.GENERAL_SETTING_Z_ROT){
					fnew = Float.parseFloat(wb.getFieldText(WidgetBuilder.TE_OBJ_Z_ROT));
					Float freverse = (oa.z_rot - oa.init_z_rot)*-1;
					oa.obj3D.rotateZ(freverse);
					oa.obj3D.rotateZ(fnew);
					oa.z_rot = fnew;
				}
			}
					
		}catch(Exception e){
			System.err.println("PropertyChanger::updateLocation() " + e.getMessage());
			e.printStackTrace();
		}
	}


	public static void updateLocation(HashMap hs, SimpleVector vdelta){
		try{
			if(hs != null){
				Iterator it = hs.values().iterator();
				while (it.hasNext()) {
					ObjectAttributes oa = (ObjectAttributes)it.next();
					oa.obj3D.translate(vdelta);
					oa.x = oa.obj3D.getTransformedCenter().x;
					oa.y = oa.obj3D.getTransformedCenter().y;
					oa.z = oa.obj3D.getTransformedCenter().z;
				}	
			}	

		}catch(Exception e){
			System.err.println("PropertyChanger::updateLocation() " + e.getMessage());
			e.printStackTrace();
		}
	}
	public static void updateRotation(HashMap hs, int imode, float fdelta){
		try{
			if(hs != null){
				Iterator it = hs.values().iterator();
				while (it.hasNext()) {
					ObjectAttributes oa = (ObjectAttributes)it.next();
					
					if(imode == WidgetBuilder.GENERAL_SETTING_X_ROT){
						oa.obj3D.rotateX(fdelta);
						oa.x_rot += fdelta;
					}
					if(imode == WidgetBuilder.GENERAL_SETTING_Y_ROT){
						oa.obj3D.rotateY(fdelta);
						oa.y_rot += fdelta;
					}
					if(imode == WidgetBuilder.GENERAL_SETTING_Z_ROT){
						oa.obj3D.rotateZ(fdelta);
						oa.z_rot += fdelta;
					}
				}	
			}	

		}catch(Exception e){
			System.err.println("PropertyChanger::updateRotation() " + e.getMessage());
			e.printStackTrace();
		}
	}
	public static void updateTransparency(HashMap hs, int idelta){
		try{
			if(hs != null){
				Iterator it = hs.values().iterator();
				while (it.hasNext()) {
					ObjectAttributes oa = (ObjectAttributes)it.next();
					oa.transparency += idelta;
					oa.obj3D.setTransparency(oa.transparency);
				}	
			}	

		}catch(Exception e){
			System.err.println("PropertyChanger::updateTransparency() " + e.getMessage());
			e.printStackTrace();
		}

	}

	public static void debugSelected(HashMap hs){
		try{
			if(hs != null){
				Iterator it = hs.values().iterator();
				while (it.hasNext()) {
					ObjectAttributes oa = (ObjectAttributes)it.next();
					System.out.println("PropertyChanger::debugSelected() element [" + oa.obj3D.getName() + "] found.");
				}	
			}else{
				System.out.println("PropertyChanger::debugSelected() HashMap is null!");
			}

		}catch(Exception e){
			System.err.println("PropertyChanger::debugSelected() " + e.getMessage());
			e.printStackTrace();
		}

	}

	public static void updateScale(HashMap hs, float fscale){
		try{
			if(hs != null){
				Iterator it = hs.values().iterator();
				while (it.hasNext()) {
					ObjectAttributes oa = (ObjectAttributes)it.next();
					oa.scale += fscale;
					System.out.println("PropertyChanger::updateScale() setting [" + oa.obj3D.getName() + "] scale to " + oa.scale);
					oa.obj3D.setScale(oa.scale);
				}	
			}	

		}catch(Exception e){
			System.err.println("PropertyChanger::updateScale() " + e.getMessage());
			e.printStackTrace();
		}
	}

}